package lel.flummi.skilloverlay.api.records;

import java.util.HashMap;
import java.util.List;

import lel.flummi.skilloverlay.utils.SkillInfo;

public record Skills(
		HashMap<String, Skill> skills) {
	public record Skill(
			String name,
			int maxLevel,
			List<Level> levels) {
		public record Level(
				int level,
				float totalExpRequired) {
		}

		public SkillInfo getLevel(float exp) {
			LevelObj levelObj = new LevelObj();
			levelObj.totalXp = exp;
			levelObj.maxLevel = 60;
			boolean cumulative = true;

			for (int level = 0; level < this.levels().size(); level++) {
				float levelXp = this.levels().get(level).totalExpRequired;

				if (levelXp > exp) {
					if (cumulative) {
						float previous = level > 0 ? this.levels().get(level - 1).totalExpRequired : 0;
						levelObj.maxXpForLevel = (levelXp - previous);
						levelObj.level = 1 + level + (exp - levelXp) / levelObj.maxXpForLevel;
					} else {
						levelObj.maxXpForLevel = levelXp;
						levelObj.level = level + exp / levelXp;
					}

					if (levelObj.level > 60) {
						levelObj.level = 60;
						levelObj.maxed = true;
					}

					return new SkillInfo(levelObj.level, levelObj.totalXp, (levelObj.level % 1) * levelObj.maxXpForLevel, levelObj.maxXpForLevel);
				}
				else {
					if (!cumulative) {
						exp -= levelXp;
					}
				}
			}

			levelObj.level = Math.min(this.levels().size(), 60);
			levelObj.maxed = true;

			return new SkillInfo(levelObj.level, levelObj.totalXp, (levelObj.level % 1) * levelObj.maxXpForLevel, levelObj.maxXpForLevel);
		}

		public static class LevelObj {
			public float level = 0;
			public float maxXpForLevel = 0;
			public boolean maxed = false;
			public int maxLevel;
			public float totalXp;
		}

	}
}
